{  PC-Eprom Burner
	Version 2.00
	v2.00 Started : 7/10/95
	Last edited   : 29/10/95
	Copyright(c) 1993/1995 David L. Jones
	Advanced R&D Solutions (R)
	Written in Borland Pascal 7.0
	For use with PCEB Rev 1.1
}
{$N-,E-}{										TURN FLOATING POINT OFF}
{$I-}{                                 TURN ON I/O CHECKING}
{$Q-}{                                 TURN OFF OVERFLOW CHECKING}
PROGRAM PCEB;

USES CRT, DOS, PCEBIL10;

CONST
	TEMPFILE='TEMPFILE.EPB';            {DATA TEMP FILE FILENAME}
	TYPEFILE='TYPEFILE.EPB';            {DEVICE TYPE LIBRARY FILENAME}
	CONFIGFILE='CONFIG.EPB';            {CONFIGURATION FILE FILENAME}
	MAXD=8191;                     		{MAXIMUM DATA BUFFER BLOCK SIZE}
	MAXTYPE=250;                        {MAXIMUM NUMBER OF DEVICE TYPES}

TYPE
	DATATYPE=RECORD                     {MEMORY DATA BUFFER ARRAY}
		D:ARRAY[0..MAXD] OF BYTE;
	END;
	EPROMTYPE=RECORD                    {DEVICE TYPE}
		DESC:STRING[35];                 {DESCRIPTION OF EPROM TYPE NUMBER}
		VCC:BYTE;                        {MAXIMUM VCC VOLTAGE}
		PULSE:WORD;                      {PULSE LENGTH}
		SIZE:BYTE;                       {EPROM SIZE ID NUMBER}
		RETRY:BYTE;                      {NUMBER OF WRITE RETRYS BEFORE ABORT}
		FINAL:BYTE;                      {LENGTH OF FINAL PULSE}
		ALG:BYTE;                        {PROGRAMMING ALGORITHM NUMBER}
		MAN_ID:BYTE;							{MANUFACTURERS ID CODE}
		DEVICE_ID:BYTE;						{DEVICE ID CODE}
	END;

VAR
	PULSE:INTEGER;                      {PROGRAM PULSE LENGTH IN mSec}
	VCC:BYTE;                           {MAXIMUM VCC VOLTAGE}
	ALG:BYTE;                           {PROGRAMMING ALGORITHM NUMBER}
	RETRY:BYTE;                         {NUMBER OF WRITE RETRYS BEFORE ABORT}
	FINAL:BYTE;                         {FINAL PULSE LENGTH}
	ID_MANUF:BYTE; 							{MANUFACTURERS ID BYTE}
	ID_DEVICE:BYTE;							{DEVICE ID BYTE}
	FILENAME:STRING;                    {FILENAME FOR DATA FILE}
	ED:ARRAY[0..MAXTYPE] OF EPROMTYPE;  {ARRAY OF DATA TYPES}
	ETYPE:INTEGER;                      {CURRENT EPROM TYPE NUMBER}
	REG:REGISTERS;                      {REGISTERS FOR INTERUPT CALLING}
	BANK:BYTE;                          {CURRENT DATA BANK NUMBER}
	SIZE:LONGINT;                       {CURRENT EPROM SIZE IN BYTES}
	FILETYPE:FILE;                      {FILE TYPE VARIABLE}
	TEMPTYPE:FILE OF DATATYPE;          {TEMP FILE TYPE VARIAVLE}
	EPTYPE:FILE OF EPROMTYPE;           {EPROM FILE TYPE VARIABLE}
	CONFIG:TEXT;                        {CONFIG FILE TYPE}
	ENUM:INTEGER;                       {NUMBER OF EPROM TYPES}
	LASTADDR:LONGINT;                   {LAST ADDRESS REACHED}
	STARTADDR:LONGINT;                  {START ADDRESS FOR EPROM}
	ENDADDR:LONGINT;                    {ENDING ADDRESS FOR EPROM}
	LASTMESSAGE:STRING;                 {LAST DISPLAYED MESSAGE}
	FC,FC2,BC,BC2,H1,H2:BYTE;           {FORGROUND/BACKGROUND COLORS}
	DLY:WORD;                           {DELAY FOR SMALL_DELAY PROCEDURE}
	LOOPS,LOOPS2:LONGINT;               {NUMBER OF LOOPS FOR CAL. ROUTINE}
	DISPCOUNT:BYTE;                     {VARIABLE FOR UPDATING ADDR DISPLAY}
	CRC16 : WORD;								{CRC-16 CHECKSUM VALUE}
	DATA:ARRAY[0..63] OF ^DATATYPE;		{MAIN DATA BUFFER ARRAY. 4Mbit MAX}
	SIZEDESC:ARRAY[0..19] OF STRING[4];	{DESC. FOR EACH BIT SIZE}
	ESIZE:BYTE;									{CURRENTLY SELECTED EPROM BIT SIZE}
	NUMBLOCKS:BYTE;							{NUMBER OF 8K DATA BLOCKS (64) 0-63}
	ALGNAME:ARRAY[0..20] OF STRING[30]; {ALGORITHM NAMES}
	AUTOID:BOOLEAN;							{TRUE=AUTOID ON    FALSE=AUTOID OFF


{******************************************************}
PROCEDURE WRITEXY(X,Y:BYTE; S:STRING);
BEGIN
	GOTOXY(X,Y);
	WRITE(S);
END;


{******************************************************}
FUNCTION REPSTR(C:CHAR; NUM:BYTE) : STRING;
VAR D:BYTE;
	S:STRING;
BEGIN
	S:='';
	FOR D:=1 TO NUM DO
		S:=S+C;
	REPSTR:=S;
END;


{******************************************************}
PROCEDURE COLOR(FC,BC:BYTE);
BEGIN
	TEXTCOLOR(FC);
	TEXTBACKGROUND(BC);
END;


{******************************************************}
PROCEDURE DISPLAY_MESSAGE(S:STRING);
VAR CH:CHAR;
BEGIN
	TEXTBACKGROUND(BC);
	GOTOXY(1,24);											{CLEAR ANY OLD MESSAGE}
	CLREOL;

	IF S<>'' THEN BEGIN
		COLOR(FC2, BC2);
		GOTOXY(40-ROUND(LENGTH(S)/2),24);
		WRITE(S);
	END;
	COLOR(FC,BC);
END;


{******************************************************
GLOBAL VARIABLE 'LOOPS' IS CALCULATED TO PROVIDE A FOR LOOP FROM ZERO
THAT PROVIDES A 100uS DELAY}
{******************************************************}
PROCEDURE CALIBRATE_TIMER;
VAR
	T:REAL;
	STARTTIME, NEWTIME, F, C, D:LONGINT;
BEGIN
	F:=1000;
	D:=0;
	DISPLAY_MESSAGE('');
	DISPLAY_MESSAGE('Calibrating Timer...');
	STARTTIME:=LONGINT(PTR($40, $6C)^);        	{GET ROM BIOS TIMER TICK COUNT}
	REPEAT
		FOR C:=1 TO F DO;                    	{EXECUTE THE DUMMY LOOP}
		NEWTIME:=LONGINT(PTR($40, $6C)^);     	{GET ROM BIOS TIMER TICK COUNT}
		INC(D);
	UNTIL (NEWTIME-STARTTIME)>54.62;            	{WAIT 3 SECONDS}
	DISPLAY_MESSAGE('');
	T:=(NEWTIME-STARTTIME)/18.206482;        	{CALCULATE NUMBER OF SECONDS}
	LOOPS:=ROUND(100e-6/(T/(F*D)));         	{CALCULATE 100uS}
	LOOPS2:=ROUND(5e-6/(T/(F*D)));         	{CALCULATE 5uS}
END;


{******************************************************}
{ CREATES A 100us DELAY USING THE GLOBAL VARIABLE 'LOOPS'}
{******************************************************}
PROCEDURE DELAY100;
VAR C:LONGINT;
BEGIN
	FOR C:=1 TO LOOPS DO;
END;


{******************************************************}
{ CREATES A 5us DELAY USING THE GLOBAL VARIABLE 'LOOPS2'}
{******************************************************}
PROCEDURE DELAY5;
VAR C:LONGINT;
BEGIN
	FOR C:=1 TO LOOPS2 DO;
END;


{******************************************************}
{ CALCULATES A STANDARD CRC-16 CHECKSUM USING THE CCITT POLYNOMIAL}
{ THIS ROUTINE USES THE LOOKUP TABLE METHOD FOR INCREASED SPEED}
{******************************************************}
PROCEDURE CALCULATE_CRC;
CONST
	TABLE : ARRAY[0..255] OF WORD =
($0000, $1021, $2042, $3063, $4084, $50A5, $60C6, $70E7,
 $8108, $9129, $A14A, $B16B, $C18C, $D1AD, $E1CE, $F1EF,
 $1231, $0210, $3273, $2252, $52B5, $4294, $72F7, $62D6,
 $9339, $8318, $B37B, $A35A, $D3BD, $C39C, $F3FF, $E3DE,
 $2462, $3443, $0420, $1401, $64E6, $74C7, $44A4, $5485,
 $A56A, $B54B, $8528, $9509, $E5EE, $F5CF, $C5AC, $D58D,
 $3653, $2672, $1611, $0630, $76D7, $66F6, $5695, $46B4,
 $B75B, $A77A, $9719, $8738, $F7DF, $E7FE, $D79D, $C7BC,
 $48C4, $58E5, $6886, $78A7, $0840, $1861, $2802, $3823,
 $C9CC, $D9ED, $E98E, $F9AF, $8948, $9969, $A90A, $B92B,
 $5AF5, $4AD4, $7AB7, $6A96, $1A71, $0A50, $3A33, $2A12,
 $DBFD, $CBDC, $FBBF, $EB9E, $9B79, $8B58, $BB3B, $AB1A,
 $6CA6, $7C87, $4CE4, $5CC5, $2C22, $3C03, $0C60, $1C41,
 $EDAE, $FD8F, $CDEC, $DDCD, $AD2A, $BD0B, $8D68, $9D49,
 $7E97, $6EB6, $5ED5, $4EF4, $3E13, $2E32, $1E51, $0E70,
 $FF9F, $EFBE, $DFDD, $CFFC, $BF1B, $AF3A, $9F59, $8F78,
 $9188, $81A9, $B1CA, $A1EB, $D10C, $C12D, $F14E, $E16F,
 $1080, $00A1, $30C2, $20E3, $5004, $4025, $7046, $6067,
 $83B9, $9398, $A3FB, $B3DA, $C33D, $D31C, $E37F, $F35E,
 $02B1, $1290, $22F3, $32D2, $4235, $5214, $6277, $7256,
 $B5EA, $A5CB, $95A8, $8589, $F56E, $E54F, $D52C, $C50D,
 $34E2, $24C3, $14A0, $0481, $7466, $6447, $5424, $4405,
 $A7DB, $B7FA, $8799, $97B8, $E75F, $F77E, $C71D, $D73C,
 $26D3, $36F2, $0691, $16B0, $6657, $7676, $4615, $5634,
 $D94C, $C96D, $F90E, $E92F, $99C8, $89E9, $B98A, $A9AB,
 $5844, $4865, $7806, $6827, $18C0, $08E1, $3882, $28A3,
 $CB7D, $DB5C, $EB3F, $FB1E, $8BF9, $9BD8, $ABBB, $BB9A,
 $4A75, $5A54, $6A37, $7A16, $0AF1, $1AD0, $2AB3, $3A92,
 $FD2E, $ED0F, $DD6C, $CD4D, $BDAA, $AD8B, $9DE8, $8DC9,
 $7C26, $6C07, $5C64, $4C45, $3CA2, $2C83, $1CE0, $0CC1,
 $EF1F, $FF3E, $CF5D, $DF7C, $AF9B, $BFBA, $8FD9, $9FF8,
 $6E17, $7E36, $4E55, $5E74, $2E93, $3EB2, $0ED1, $1EF0
 );
VAR CRC:WORD;
	C,B,A:LONGINT;

BEGIN
	CRC:=0;
	B:=0;
	A:=0;

	FOR C:=0 TO ENDADDR DO         			{ABSOLUTE ADDRESS COUNTER}
		IF C>=STARTADDR THEN BEGIN
			CRC:=(CRC SHL 8) XOR TABLE[(CRC SHR 8) XOR DATA[B]^.D[A]];
			{***** STANDARD CCITT POLYNOMIAL ROUTINE FOR CRC-16 *****}

			INC(A);									{INCREMENT BLOCK OFFSET COUNTER}

			IF A>MAXD THEN BEGIN             {IF END OF BLOCK}
				A:=0;                         {RESET BLOCK OFFSET COUNTER}
				INC(B);                       {INCREMENT BLOCK COUNTER}
			END;
		END;
	CRC16:=CRC;
END;


{******************************************************}
PROCEDURE BEEP;
BEGIN
	SOUND(800);
	DELAY(150);
	NOSOUND;
END;


{******************************************************}
{TAKES 'NUM' AND RETURNS STRING 'NUMDIGITS' LONG WHICH IS THE HEX VAL OF NUM}
{******************************************************}
FUNCTION DEC_TO_HEX(NUM, NUMDIGITS:LONGINT):STRING;
VAR
	A:LONGINT;
	C:BYTE;
	S:STRING;
BEGIN
	S:='';
	FOR C:=7 DOWNTO 0 DO BEGIN
		A:=NUM SHR (4*C);
		A:=A AND $0F;
		CASE A OF
			0..9 : S:=S+CHR(48+A);
			10..15 : S:=S+CHR(87+A);
		END;
	END;
	DEC_TO_HEX:=COPY(S,9-NUMDIGITS,8)
END;


{******************************************************}
FUNCTION HEX_TO_DEC(S:STRING):LONGINT;
VAR V,F:LONGINT;
	SUB:STRING;
	C,D,E,CODE:INTEGER;
BEGIN
	F:=0;
	FOR C:=0 TO LENGTH(S)-1 DO BEGIN
		SUB:=COPY(S, LENGTH(S)-C, 1);
		IF SUB='0' THEN D:=0;
		IF SUB='1' THEN D:=1;
		IF SUB='2' THEN D:=2;
		IF SUB='3' THEN D:=3;
		IF SUB='4' THEN D:=4;
		IF SUB='5' THEN D:=5;
		IF SUB='6' THEN D:=6;
		IF SUB='7' THEN D:=7;
		IF SUB='8' THEN D:=8;
		IF SUB='9' THEN D:=9;
		IF SUB='A' THEN D:=10;
		IF SUB='B' THEN D:=11;
		IF SUB='C' THEN D:=12;
		IF SUB='D' THEN D:=13;
		IF SUB='E' THEN D:=14;
		IF SUB='F' THEN D:=15;
		V:=1;
		FOR E:=0 TO C-1 DO V:=V*16;
		F:=F+(V*D);
	END;
	HEX_TO_DEC:=F;
END;


{******************************************************}
FUNCTION FILE_EXISTS(FILENAME:STRING):BOOLEAN;
VAR PATH:STRING;
BEGIN
	GETDIR(0,PATH);
	IF FSEARCH(FILENAME,PATH)='' THEN FILE_EXISTS:=FALSE
		ELSE FILE_EXISTS:=TRUE;
END;


{******************************************************}
PROCEDURE INITIALISE_DATA_ARRAY;
VAR C:WORD;
BEGIN
	FOR C:=0 TO NUMBLOCKS DO NEW(DATA[C]);;		{DECLARE DATA ARRAY}
	SIZEDESC[0]:='';
	SIZEDESC[13]:='64K';
	SIZEDESC[14]:='128K';
	SIZEDESC[15]:='256K';
	SIZEDESC[16]:='512K';
	SIZEDESC[17]:='1M';
	SIZEDESC[18]:='2M';
	SIZEDESC[19]:='4M';
	ALGNAME[0]:='Quick Pulse';
	ALGNAME[1]:='512K Quick Pulse';
	ALGNAME[2]:='Multiple Verify Quick Pulse';
END;


{******************************************************}
PROCEDURE DISPLAY_FILES;
VAR
	T:SEARCHREC;
	DT:DATETIME;
	ROW:BYTE;
	DUMMY:CHAR;
	S1,S2,S3:STRING;
BEGIN
	T.NAME:='';
	FINDFIRST('*.*', ANYFILE, T);
	IF T.NAME='' THEN EXIT;                   {EXIT IF NO FILES}
	REPEAT
		ROW:=2;
		CLRSCR;
		GETDIR(0,S1);
		WRITELN('Directory of ',S1);
		REPEAT
			UNPACKTIME(T.TIME,DT);
			WRITEXY(1,ROW,T.NAME);
			WRITE(14,ROW,T.SIZE:10);
			GOTOXY(26,ROW);
			IF FRAC(T.SIZE/1024)=0 THEN WRITE(T.SIZE/1024:4:0,'Kb');
			GOTOXY(34,ROW);
			IF DT.DAY<10 THEN WRITE('0'); WRITE(DT.DAY,'-');
			IF DT.MONTH<10 THEN WRITE('0'); WRITE(DT.MONTH,'-',DT.YEAR);
			GOTOXY(48,ROW);
			IF DT.HOUR<10 THEN WRITE('0'); WRITE(DT.HOUR,':');
			IF DT.MIN<10 THEN WRITE('0');  WRITE(DT.MIN,':');
			IF DT.SEC<10 THEN WRITE('0');  WRITE(DT.SEC);
			INC(ROW);
			FINDNEXT(T);
		UNTIL (DOSERROR=18) OR (ROW>23);
		WRITELN;
		WRITE('Press any key for more...');
		DUMMY:=READKEY;
	UNTIL DOSERROR=18;
END;


{******************************************************}
{CLEAR ENTIRE DATA BUFFER
{******************************************************}
PROCEDURE CLEAR_ARRAY;
VAR C:BYTE;
	D:WORD;
BEGIN
	FOR C:=0 TO NUMBLOCKS DO
		FOR D:=0 TO MAXD DO DATA[C]^.D[D]:=$FF;
END;


{******************************************************}
{   LOADS THE EPROM TYPE LIBRARY FILE INTO THE 'ED' GLOBAL ARRAY}
{******************************************************}
PROCEDURE LOAD_EPROM_TYPES;
BEGIN
	ENUM:=0;
	ED[0].DESC:='No device type selected.';
	ED[0].VCC:=0;
	ED[0].PULSE:=0;
	ED[0].SIZE:=0;
	ED[0].RETRY:=0;
	ED[0].ALG:=0;
	ED[0].DEVICE_ID:=0;
	ED[0].MAN_ID:=0;
	ESIZE:=0;
	IF NOT(FILE_EXISTS(TYPEFILE)) THEN EXIT;
	FILEMODE:=0;                              {FILE IS READ ONLY}
	ASSIGN(EPTYPE, TYPEFILE);
	RESET(EPTYPE);
	IF IORESULT<>0 THEN EXIT;
	ENUM:=1;
	WHILE (ENUM<=MAXTYPE) AND NOT(EOF(EPTYPE)) AND (IORESULT=0) DO BEGIN
		READ(EPTYPE,ED[ENUM]);
		INC(ENUM);
	END;
	DEC(ENUM);
	CLOSE(EPTYPE);
END;


{******************************************************}
{   DISPLAYS THE CURRENT ADDRESS IN HEX.
	GLOBAL VARIABLE DISPCOUNT IS INCREMENTED AFTER EACH ADDRESS OPERATION.
	THE ADDRESS DISPLAY IS UPDATED IN BLOCKS OF 10, TO SPEED THINGS UP}
{******************************************************}
PROCEDURE DISPLAY_ADDRESS(C:LONGINT);
BEGIN
	INC(DISPCOUNT);
	IF DISPCOUNT>=10 THEN BEGIN
		WRITEXY(48,16,DEC_TO_HEX(C,6));
		DISPCOUNT:=0;
	END;
END;


{******************************************************}
{ RETURNS TRUE IF AN EPROM IS LOADED}
{******************************************************}
FUNCTION DEVICE_LOADED:BOOLEAN;
BEGIN
	DEVICE_LOADED:=FALSE;
	IF ETYPE=0 THEN BEGIN
		LASTMESSAGE:='No device type selected.';
		BEEP;
	END
	ELSE DEVICE_LOADED:=TRUE;
END;


{******************************************************}
{ RETURNS 2^(BITS)}
{******************************************************}
FUNCTION RETURN_SIZE(BITS:BYTE):LONGINT;
VAR C:LONGINT;
	D:BYTE;
BEGIN
	C:=1;
	FOR D:=1 TO BITS DO C:=C*2;
	RETURN_SIZE:=C;
END;


{******************************************************}
PROCEDURE DISPLAY_SETTINGS;
VAR S:STRING;
BEGIN
	COLOR(FC,BC);
	LOWVIDEO;  WRITEXY(30,4,'Device Type : ');
	HIGHVIDEO; WRITE  (ED[ETYPE].DESC);
	LOWVIDEO;  WRITEXY(30,5,'Device Size : ');
	HIGHVIDEO; WRITE  ((SIZE*8) DIV 1024,' Kbits / ',SIZE DIV 1024:3,' Kbytes');
	LOWVIDEO;  WRITEXY(30,6,'Vcc Voltage : ');
	HIGHVIDEO; WRITE  (VCC,'V  ');
	LOWVIDEO;  WRITEXY(30,7,'Pulse Width : ');
	HIGHVIDEO;
	IF PULSE=0 THEN WRITE('0.1ms') ELSE WRITE(PULSE,' ms ');
	LOWVIDEO;  WRITEXY(30,8,'Pulse Retry : ');
	HIGHVIDEO; WRITE  (RETRY,' times');
	LOWVIDEO;  WRITEXY(30,9,'Final Pulse : ');
	HIGHVIDEO;
	IF FINAL=0 THEN WRITE ('None') ELSE WRITE (FINAL,' * retry');
	LOWVIDEO;  WRITEXY(30,10,'Algorithm   : ');
	HIGHVIDEO;
	CASE ALG OF
		0: WRITE(ALGNAME[ALG]);
		1: WRITE(ALGNAME[ALG]);
		2: WRITE(ALGNAME[ALG]);
	END;
	IF FILENAME='' THEN S:='No file selected' ELSE S:=FILENAME;
	LOWVIDEO;  WRITEXY(30,11,'Filename    : '); HIGHVIDEO; WRITE(S);
	LOWVIDEO;  WRITEXY(30,12,'Free space  : ');
	HIGHVIDEO; WRITE  (DISKFREE(0),' bytes');

	LOWVIDEO;  WRITEXY(30,14,'Manufacturer ID : ');
	HIGHVIDEO; WRITE  (DEC_TO_HEX(ID_MANUF, 2));
	LOWVIDEO;  WRITEXY(55,14,'Device ID : ');
	HIGHVIDEO; WRITE  (DEC_TO_HEX(ID_DEVICE,2));
	LOWVIDEO;  WRITEXY(30,15,'Address Range   : ');
	HIGHVIDEO; WRITE  (DEC_TO_HEX(0, 6),#26,DEC_TO_HEX(SIZE-1,6));
	LOWVIDEO;  WRITEXY(30,16,'Current Address : ');
	HIGHVIDEO; WRITE  (DEC_TO_HEX(0, 6));
	LOWVIDEO;  WRITEXY(30,17,'Last    Address : ');
	HIGHVIDEO; WRITE  (DEC_TO_HEX(LASTADDR, 6));
	LOWVIDEO;  WRITEXY(30,18,'Start   Address : ');
	HIGHVIDEO; WRITE  (DEC_TO_HEX(STARTADDR, 6),'   ');
	LOWVIDEO;  WRITEXY(30,19,'End     Address : ');
	HIGHVIDEO; WRITE  (DEC_TO_HEX(ENDADDR, 6),'   ');
	LOWVIDEO;  WRITEXY(30,20,'CRC-16 Checksum : ');
	HIGHVIDEO; WRITE  (DEC_TO_HEX(CRC16, 4),'   ');
	TEXTCOLOR(H2);
	WRITEXY(5,22,'Last Message : '+LASTMESSAGE);
	TEXTCOLOR(FC);
	CLREOL;
	WRITEXY(21,19,DEC_TO_HEX(BASEADDR,3));
	IF AUTOID THEN S:='ON ' ELSE S:='OFF';
	WRITEXY(21,17,S);
END;


{*************************************************************}
PROCEDURE SELECT_TYPE(NUM:WORD);
BEGIN

	ETYPE:=NUM;
	ESIZE:=ED[NUM].SIZE;
	SIZE :=RETURN_SIZE(ED[NUM].SIZE);
	PULSE:=ED[NUM].PULSE;
	VCC  :=ED[NUM].VCC;
	RETRY:=ED[NUM].RETRY;
	FINAL:=ED[NUM].FINAL;
	ALG  :=ED[NUM].ALG;
	ID_MANUF:=ED[NUM].MAN_ID;
	ID_DEVICE:=ED[NUM].DEVICE_ID;
	STARTADDR:=0;
	ENDADDR:=SIZE-1;
END;


{******************************************************}
{  COPIES GLOBAL VAR 'FILENAME' INTO TYPED FILE 'TEMPFILE'
	RETURNS TRUE IF SUCESSFULL
	IF ASK IS TRUE, THEN THE USER IS ASKED TO VERIFY THE LOAD}
{******************************************************}
FUNCTION LOAD_FILE(ASK:BOOLEAN):BOOLEAN;
VAR C, RESULT:WORD;
BEGIN
	LOAD_FILE:=FALSE;
	IF (FILENAME='') OR (ETYPE=0) THEN BEGIN
		LASTMESSAGE:='No filename or device selected.';
		EXIT;
	END;
	IF DISKFREE(0)<(SIZE+1000) THEN BEGIN
		LASTMESSAGE:='Not enough disk space.';
		EXIT;
	END;
	IF NOT FILE_EXISTS(FILENAME) THEN BEGIN
		LASTMESSAGE:='Filename not found.';
		EXIT;
	END;
	IF ASK THEN BEGIN
		LASTMESSAGE:='Load the file using current options, (Y/N) ?';
		DISPLAY_SETTINGS;
		IF UPCASE(READKEY)<>'Y' THEN BEGIN
			LASTMESSAGE:='';
			EXIT;
		END;
	END;
	DISPLAY_MESSAGE('');
	CLEAR_ARRAY;                		{ERASE BUFFER}
	FILEMODE:=0;                     {MAKE FILE READ ONLY}
	ASSIGN(FILETYPE, FILENAME);      {ASSIGN THE FILE TO BE COPIED}
	RESET(FILETYPE, MAXD+1);         {OPEN FILE TO BE COPIED, RECSIZE=MAXD+1}
	IF IORESULT<>0 THEN BEGIN
		CLOSE(FILETYPE);
		LOAD_FILE:=FALSE;
		LASTMESSAGE:='Error opening file';
		BEEP;
		EXIT;
	END;
	DISPLAY_MESSAGE('Loading file. Please wait...');
	C:=0;                                     {BLOCK BUFFER COUNTER}
	CLEAR_ARRAY;                           	{CLEAR THE DATA BUFFER}
	FOR C:=0 TO NUMBLOCKS DO BEGIN
		BLOCKREAD(FILETYPE, DATA[C]^.D, 1, RESULT);     {READ IN A BLOCK OF DATA OR LESS}
		IF (IORESULT>0) OR (RESULT<1) THEN BREAK;
	END;
	LOAD_FILE:=TRUE;
	LASTMESSAGE:='File loaded OK.';
	DISPLAY_MESSAGE('');
	IF ASK THEN BEEP;
END;


{******************************************************}
{  COPIES DATA FROM THE BUFFER INTO 'FILENAME'
	RETURNS TRUE IF SUCESSFULL}
{******************************************************}
FUNCTION SAVE_FILE : BOOLEAN;
VAR B,C:LONGINT;
BEGIN
	SAVE_FILE:=FALSE;
	IF (FILENAME='') OR (ETYPE=0) THEN BEGIN
		LASTMESSAGE:='No filename or device selected.';
		BEEP;
		EXIT;
	END;

	IF DISKFREE(0)<(SIZE+1000) THEN BEGIN           {LEAVE AT LEAST 1KB}
		LASTMESSAGE:='Not enough disk space.';
		BEEP;
		EXIT;
	END;

	IF NOT FILE_EXISTS(TEMPFILE) THEN EXIT;   {EXIT IF NO TEMP FILE}

	IF FILE_EXISTS(FILENAME) THEN BEGIN
		BEEP;
		DISPLAY_MESSAGE('File already exists, Overwrite (Y/N) ?');
		IF  UPCASE(READKEY)<>'Y' THEN
			BEGIN DISPLAY_MESSAGE(''); EXIT; END;
	END;

	FILEMODE:=2;                           {FILE IS READ/WRITE}
	ASSIGN(TEMPTYPE, FILENAME);            {ASSIGN THE FILE}
	REWRITE(TEMPTYPE);                     {CREATE THE FILE TO BE COPIED}

	IF IORESULT>0 THEN BEGIN
		CLOSE(TEMPTYPE);
		LASTMESSAGE:='Error opening filename.';
		BEEP;
		EXIT;
	END;

	DISPLAY_MESSAGE('Saving file. Please wait...');

	FOR B:=0 TO NUMBLOCKS DO BEGIN
		WRITE(TEMPTYPE, DATA[B]^);         						{WRITE THE BLOCK}
		IF (((B+1)*MAXD)>SIZE) OR (IORESULT<>0) THEN BREAK;
	END;

	CLOSE(TEMPTYPE);
	SAVE_FILE:=TRUE;
	LASTMESSAGE:='File saved OK.';
	DISPLAY_MESSAGE('');
	BEEP;
END;


{******************************************************}
PROCEDURE LOAD_SETUP;
VAR AID:BYTE;
BEGIN
	IF NOT FILE_EXISTS(CONFIGFILE) THEN EXIT;
	FILEMODE:=0;                              {FILE IS READ ONLY}
	ASSIGN(CONFIG,CONFIGFILE);
	RESET(CONFIG);
	READLN(CONFIG, BASEADDR);
	READLN(CONFIG, ETYPE);
	READLN(CONFIG, FILENAME);
	READLN(CONFIG, AID);
	CLOSE(CONFIG);

	IF ETYPE>ENUM THEN BEGIN
		ETYPE:=0;
		EXIT;
	END;

	IF AID=0 THEN AUTOID:=FALSE ELSE AUTOID:=TRUE;

	SELECT_TYPE(ETYPE);

	IF FILENAME<>'' THEN LOAD_FILE(FALSE)
	ELSE CLEAR_ARRAY;
END;


{******************************************************}
PROCEDURE SAVE_SETUP;
BEGIN
	FILEMODE:=2;                     {FILE IS READ/WRITE}
	ASSIGN(CONFIG,CONFIGFILE);
	REWRITE(CONFIG);
	WRITELN(CONFIG, BASEADDR);
	WRITELN(CONFIG, ETYPE);
	WRITELN(CONFIG, FILENAME);
	IF AUTOID THEN WRITELN(CONFIG, 1) ELSE WRITELN(CONFIG, 0);
	CLOSE(CONFIG);
END;


{******************************************************}
{ RETURNS A FILENAME SPECIFIED BY THE USER}
{******************************************************}
FUNCTION GET_FILENAME:STRING;
VAR
   C:BYTE;
   CH:CHAR;
	S:STRING;
BEGIN
	COLOR(FC2,BC2);
	GOTOXY(44,11); WRITE(REPSTR(' ',35));
	S:='';
	CH:=' ';
	REPEAT
		GOTOXY(44,11);
      WRITE(S,' ');
      CH:=UPCASE(READKEY);
      CASE CH OF
         #33..#95: IF LENGTH(S)<35 THEN S:=S+CH;
			#8: S:=COPY(S,1,LENGTH(S)-1);
      END;
	UNTIL (CH=#13) OR (CH=#27);
	IF CH=#13 THEN BEGIN
		GET_FILENAME:=S;
		LASTMESSAGE:='Filename changed.';
		END
	ELSE GET_FILENAME:=FILENAME;

	TEXTBACKGROUND(BC);
	GOTOXY(44,11);
	WRITE(REPSTR(' ',35));
END;


{******************************************************}
PROCEDURE DISPLAY_EDITOR(B,WINSTART:LONGINT;ROW,CURSOR:INTEGER;WIN:BOOLEAN);
VAR
   A,C,CC:LONGINT;
   CH:CHAR;
BEGIN
   FOR C:=0 TO 15 DO BEGIN
		IF (ROW<0) OR (ROW=C) THEN BEGIN
			GOTOXY(1,C+3);
         A:=WINSTART+(C*16)+((MAXD+1)*B);       {CALCULATE ADDRESS}
         WRITE(' ', DEC_TO_HEX(A,6), ' ');
			FOR CC:=0 TO 15 DO BEGIN
				COLOR(FC,BC);
            IF CC=8 THEN WRITE('  ') ELSE WRITE(' ');
				IF (CURSOR=CC) AND (NOT WIN) THEN COLOR(FC2,BC2);
				WRITE(DEC_TO_HEX(DATA[B]^.D[WINSTART+(C*16)+CC],2));
			END;
			COLOR(FC,BC);
			WRITE('  ');
			FOR CC:=0 TO 15 DO BEGIN

				CH:=CHR(DATA[B]^.D[WINSTART+(C*16)+CC]);
				IF (CURSOR=CC) AND (WIN) THEN COLOR(FC2,BC2)
					ELSE COLOR(FC,BC);
				CASE CH OF
					CHR(0)..CHR(31), CHR(127): WRITE(CHR(22));
				ELSE
					WRITE(CH);                    {DISPLAY THE ASCII CHARACTER}
				END;
			END;
			COLOR(FC,BC);
			WRITE(' ');
		END;
	END;
END;

{******************************************************}
PROCEDURE TYPE_LIBRARY;
CONST MAXLINES=15;
VAR
	CUR, CUR2 ,TEMP: BYTE;
	NUM,START,C:INTEGER;
	CH:CHAR;
	S:STRING;
	T:EPROMTYPE;
BEGIN
	CLRSCR;
	CUR:=1; CUR2:=1; START:=1;
	TEXTCOLOR(H1);
	WRITELN('         Device Description            Size Pulse Vcc Retry Final Alg Manf Devc');
	TEXTCOLOR(FC);
	WRITE  ('ͻ');
	GOTOXY(1,18);
	WRITE  ('ͼ');
	TEXTCOLOR(H2);
	WRITELN('      Use arrow keys and PgUp/PgDn to scroll cursor');
	WRITELN;
	WRITELN('    S - Save Type Library         ENTER - Select device and exit');
	WRITELN('    A - Add a new device type       ESC - Exit without selecting');
	WRITELN('    E - Edit description            +/- - Change numeric value');
	TEXTCOLOR(FC);
	REPEAT
		FOR NUM:=1 TO 15 DO BEGIN
			IF (START+NUM-2)<ENUM THEN
				T:=ED[START+NUM-1]
			ELSE
				BEGIN
				T.DESC:='';
				T.PULSE:=0;
				T.VCC:=0;
				T.SIZE:=0;
				T.ALG:=0;
				T.FINAL:=0;
				T.RETRY:=0;
				T.MAN_ID:=0;
				T.DEVICE_ID:=0;
			END;
			COLOR(FC,BC);
			GOTOXY(1,NUM+2);
			WRITE('');

			IF (CUR=NUM) AND (CUR2=1) THEN COLOR(FC2,BC2);
			WRITE(COPY(T.DESC,1,35):37);                  	{DISPLAY DESC}
			COLOR(FC,BC);
			WRITE('');

			IF (CUR=NUM) AND (CUR2=2) THEN COLOR(FC2,BC2);
			WRITE(SIZEDESC[T.SIZE]:4);               			{DISPLAY SIZE}
			COLOR(FC,BC);
			WRITE('');

			IF (CUR=NUM) AND (CUR2=3) THEN COLOR(FC2,BC2);
			IF (START+NUM-1)>ENUM THEN WRITE('    ')
				ELSE IF T.PULSE=0 THEN WRITE (' 0.1')
					ELSE WRITE(T.PULSE:4);                    {DISPLAY PULSE}
			COLOR(FC,BC);
			WRITE(' ');

			IF (CUR=NUM) AND (CUR2=4) THEN COLOR(FC2,BC2);
			IF (START+NUM-1)>ENUM THEN WRITE('  ') ELSE WRITE(T.VCC:2); {DISPLAY VCC}
			COLOR(FC,BC);
			WRITE(' ');

			IF (CUR=NUM) AND (CUR2=5) THEN COLOR(FC2,BC2);
			IF (START+NUM-1)>ENUM THEN WRITE('    ') ELSE WRITE(T.RETRY:4);
			COLOR(FC,BC);     							         {DISPLAY RETRY}
			WRITE(' ');

			IF (CUR=NUM) AND (CUR2=6) THEN COLOR(FC2,BC2);
			IF (START+NUM-1)>ENUM THEN WRITE('    ')
				ELSE WRITE(T.FINAL:4);								{DISPLAY FINAL}
			COLOR(FC,BC);
			WRITE(' ');

			IF (CUR=NUM) AND (CUR2=7) THEN COLOR(FC2,BC2);
			IF (START+NUM-1)>ENUM THEN WRITE('  ')
				ELSE WRITE(T.ALG:2);									{DISPLAY ALGORITHM}
			COLOR(FC,BC);
			WRITE(' ');

			IF (CUR=NUM) AND (CUR2=8) THEN COLOR(FC2,BC2);
			IF (START+NUM-1)>ENUM THEN WRITE('   ')
				ELSE WRITE(DEC_TO_HEX(T.MAN_ID,2):3);				{DISPLAY MANUFACTUR}
			COLOR(FC,BC);
			WRITE(' ');

			IF (CUR=NUM) AND (CUR2=9) THEN COLOR(FC2,BC2);
			IF (START+NUM-1)>ENUM THEN WRITE('   ')
				ELSE WRITE(DEC_TO_HEX(T.DEVICE_ID,2):3);	 			{DISPLAY DEVICE ID}
			COLOR(FC,BC);
			WRITE(' ');

		END;
		CH:=UPCASE(READKEY);
		CASE CH OF
			#0: BEGIN
				CH:=READKEY;
				CASE CH OF
					'P': IF (CUR<NUM) AND ((CUR+START-1)<ENUM) THEN
                        INC(CUR);                              {DOWN ARROW}
               'H': IF CUR>1 THEN DEC(CUR);                    {UP ARROW}
               'Q': IF ENUM>(START+MAXLINES-1) THEN BEGIN      {PAGE DOWN}
								CUR:=1;
								INC(START,MAXLINES);
                    END;
               'I': IF START>1 THEN BEGIN                      {PAGE UP}
                        DEC(START,MAXLINES);
                        IF START<1 THEN START:=1;
                    END;
					'M': IF CUR2<9 THEN INC(CUR2);                  {RIGHT ARROW}
               'K': IF CUR2>1 THEN DEC(CUR2);                  {LEFT ARROW}
				END;
         END;
         'S': BEGIN
            ASSIGN(EPTYPE,TYPEFILE);
            REWRITE(EPTYPE);        {ERASE THE OLD FILE, IT'S IN MEMORY ARRAY}
            IF IORESULT=0 THEN BEGIN
               FOR C:=1 TO ENUM DO WRITE(EPTYPE, ED[C]);
					DISPLAY_MESSAGE('Type File saved OK');
					END
				ELSE DISPLAY_MESSAGE('Error saving Type file');
				CLOSE(EPTYPE);
			END;
			'A': IF ENUM<MAXTYPE THEN BEGIN
						INC(ENUM);
						ED[ENUM].DESC:='No Description';
						ED[ENUM].VCC:=5;
						ED[ENUM].PULSE:=1;
						ED[ENUM].SIZE:=14;
						ED[ENUM].ALG:=0;
						ED[ENUM].FINAL:=0;
						ED[ENUM].RETRY:=10;
						ED[ENUM].MAN_ID:=0;
						ED[ENUM].DEVICE_ID:=0;
					END
					ELSE DISPLAY_MESSAGE('Maximum number of types reached.');
			#13: BEGIN
					IF ENUM>0 THEN BEGIN
						SELECT_TYPE(START+CUR-1);
						LASTMESSAGE:='New device type selected';
						EXIT;
					END;
			END;
			'+' : BEGIN
					TEMP:=START+CUR-1;
					CASE CUR2 OF
						2: IF ED[TEMP].SIZE < 19 THEN INC(ED[TEMP].SIZE);
						3: IF ED[TEMP].PULSE < 99 THEN INC(ED[TEMP].PULSE);
						4: IF ED[TEMP].VCC=5 THEN ED[TEMP].VCC:=6;
						5: IF ED[TEMP].RETRY<99 THEN INC(ED[TEMP].RETRY);
						6: IF ED[TEMP].FINAL<99 THEN INC(ED[TEMP].FINAL);
						7: IF ED[TEMP].ALG<2 THEN INC(ED[TEMP].ALG);
						8: IF ED[TEMP].MAN_ID<255 THEN INC(ED[TEMP].MAN_ID);
						9: IF ED[TEMP].DEVICE_ID<255 THEN INC(ED[TEMP].DEVICE_ID);
					END;
			END;
			'-' : BEGIN
					TEMP:=START+CUR-1;
					CASE CUR2 OF
						2: IF ED[TEMP].SIZE >13 THEN DEC(ED[TEMP].SIZE);
						3: IF ED[TEMP].PULSE >0 THEN DEC(ED[TEMP].PULSE);
						4: IF ED[TEMP].VCC=6 THEN ED[TEMP].VCC:=5;
						5: IF ED[TEMP].RETRY >0 THEN DEC(ED[TEMP].RETRY);
						6: IF ED[TEMP].FINAL >0 THEN DEC(ED[TEMP].FINAL);
						7: IF ED[TEMP].ALG >0 THEN DEC(ED[TEMP].ALG);
						8: IF ED[TEMP].MAN_ID>0 THEN DEC(ED[TEMP].MAN_ID);
						9: IF ED[TEMP].DEVICE_ID>0 THEN DEC(ED[TEMP].DEVICE_ID);
					END;
			END;
			'E' : BEGIN
				COLOR(FC,BC);
				WRITEXY(2,CUR+2,'                               ');
				S:=ED[START+CUR-1].DESC;
				REPEAT
					WRITEXY(2,CUR+2,S+' ');
					CH:=READKEY;
					CASE CH OF
						#32..#255 : IF LENGTH(S)<35 THEN S:=S+CH;
						#8 : S:=COPY(S,1,LENGTH(S)-1);
						#13 : ED[START+CUR-1].DESC:=S;
					END;
				UNTIL (CH=#13) OR (CH=#27)
			END;
		END;
	UNTIL CH=#27;
END;

{******************************************************}
PROCEDURE EDITOR(EDIT:BOOLEAN);
VAR
	X,Y,B:INTEGER;
	WINSTART:LONGINT;
	CH,CH1,CH2:CHAR;
	CURSORWIN:BOOLEAN;
	T:BYTE;
BEGIN
	WINSTART:=0;               {START ADDRESS OF CURRENT WINDOW BLOCK}
	CURSORWIN:=FALSE;               {FALSE=HEX FIELD, TRUE=ASCII FIELD}
	X:=0; Y:=0; B:=0;
	CLRSCR;
	TEXTCOLOR(H1);
	WRITELN(' Address +  0  1  2  3  4  5  6  7   8  9  A  B  C  D  E  F        ASCII');
	TEXTCOLOR(FC);
	WRITE(''+REPSTR('',50)+'ͻ');
	GOTOXY(1,19);
	WRITE(''+REPSTR('',50)+'ͼ');
	TEXTCOLOR(H2);
	IF EDIT THEN WRITELN(' Arrow keys : Move cursor');
	WRITELN(' PgUp/PgDn  : Change page address');
	WRITELN(' F7/F8      : Block Address Up/Down');
	IF EDIT THEN BEGIN
		WRITE(' F9/F10     : Change value of byte');
		WRITELN('      For Hex window   : Enter two hex digits');
		WRITE(' TAB        : Toggle HEX/ASCII view');
		WRITELN('     For ASCII window : Press any ASCII key');
	END;
	WRITE(' ESC        : Exit to main menu');
	IF NOT EDIT THEN X:=-1;
	TEXTCOLOR(FC);
	DISPLAY_EDITOR(B,WINSTART,-1,-1,FALSE);           {DISPLAY DATA, NO CURSOR}
	REPEAT
		DISPLAY_EDITOR(B,WINSTART,Y,X,CURSORWIN);   {DISPLAY CURSOR AND IT'S ROW}
		CH:=UPCASE(READKEY);
		CASE CH OF
			CHR(0): BEGIN
				CH:=READKEY;
            CASE CH OF
               'I': BEGIN                                {PAGE UP KEY}
						DEC(WINSTART,256);
                  IF WINSTART<0 THEN WINSTART:=0;
						DISPLAY_EDITOR(B,WINSTART,-1,-1,CURSORWIN);
						END;
					'Q': BEGIN                                {PAGE DOWN KEY}
						INC(WINSTART,256);
						IF WINSTART>(MAXD-255) THEN WINSTART:=MAXD-255;
						IF WINSTART>=SIZE THEN WINSTART:=(SIZE-256);
						DISPLAY_EDITOR(B,WINSTART,-1,-1,CURSORWIN);
						END;
					'G': BEGIN                                {HOME KEY}
						WINSTART:=0;
						DISPLAY_EDITOR(B,WINSTART,-1,-1,CURSORWIN);
						END;
					'O': BEGIN                                {END KEY}
						WINSTART:=MAXD-255;
						IF WINSTART>=SIZE THEN WINSTART:=(SIZE-256);
						DISPLAY_EDITOR(B,WINSTART,-1,-1,CURSORWIN);
						END;
					'P': IF EDIT THEN BEGIN                   {DOWN ARROW}
								IF Y<15 THEN INC(Y);
								DISPLAY_EDITOR(B,WINSTART,Y-1,-1,CURSORWIN);
							END;
					'H': IF EDIT THEN BEGIN                   {UP ARROW}
								IF Y>0 THEN Y:=Y-1;
								DISPLAY_EDITOR(B,WINSTART,Y+1,-1,CURSORWIN);
							END;
					'M': IF EDIT THEN IF X<15 THEN INC(X);    {RIGHT ARROW}
					'K': IF EDIT THEN IF X>0 THEN DEC(X);     {LEFT ARROW}
					#68: IF EDIT THEN BEGIN                   {F10 KEY}
						T:=DATA[B]^.D[WINSTART+(Y*16)+X];
						IF T<255 THEN INC(T);
						DATA[B]^.D[WINSTART+(Y*16)+X]:=T; {CALC. OFFSET ADDRESS FROM X,Y}
						END;
					#67: IF EDIT THEN BEGIN                   {F9 KEY}
						T:=DATA[B]^.D[WINSTART+(Y*16)+X];
						IF T>0 THEN DEC(T);
						DATA[B]^.D[WINSTART+(Y*16)+X]:=T;
						END;
					#65: IF B>0 THEN BEGIN           {F7 KEY}
						DEC(B);
						WINSTART:=0;
						DISPLAY_EDITOR(B,WINSTART,-1,-1,CURSORWIN);
						END;
					#66: IF B<((SIZE DIV (MAXD+1))-1) THEN BEGIN   {F8 KEY}
						INC(B);
						WINSTART:=0;
						DISPLAY_EDITOR(B,WINSTART,-1,-1,CURSORWIN);
						END;
				END;
			END;
			#32..#255: IF EDIT THEN
					IF CURSORWIN THEN DATA[B]^.D[WINSTART+(Y*16)+X]:=ORD(CH)   {ANY KEY}
					ELSE BEGIN
						CH1:=UPCASE(CH);                    {GET FIRST KEY PRESS}
						CASE CH1 OF
							#48..#57,#65..#70: BEGIN         {RANGE 0-9, A-F}
								CH2:=READKEY;                 {GET SECOND KEY PRESS}
								CASE UPCASE(CH2) OF
									#48..#57,#65..#70:
										DATA[B]^.D[WINSTART+(Y*16)+X]:=HEX_TO_DEC(CH1+CH2);
								END;
							END;
						END;
					END;
			#9: CURSORWIN:=NOT CURSORWIN;						{TAB KEY}
		END;
	UNTIL CH=CHR(27);
END;


{**********************************************************************
SETS THE PIN STATES REQUIRED BY THE CURRENTLY SELECTED DEVICE TYPE
FOR READBACK MODE ONLY
**********************************************************************}
PROCEDURE SET_READ_PIN_STATE;
BEGIN
	CASE ESIZE OF
		13:BEGIN														{64K}
				PCEB_PIN26(FALSE);				{A9}
				PCEB_PIN24(FALSE);				{OE}
				PCEB_PIN30(FALSE);				{Vcc}
				PCEB_PIN3(TRUE);					{Vpp}
				PCEB_PIN31(FALSE);				{DON'T CARE}
				PCEB_PIN29(TRUE);					{PGM}
				PCEB_PIN22(FALSE);				{CE}
			END;
		14:BEGIN                                        {128K}
				PCEB_PIN26(FALSE);				{A9}
				PCEB_PIN24(FALSE);				{OE}
				PCEB_PIN30(FALSE);				{Vcc}
				PCEB_PIN3(TRUE);					{Vpp}
				PCEB_PIN31(FALSE);				{DON'T CARE}
				PCEB_PIN29(TRUE);					{PGM}
				PCEB_PIN22(FALSE);				{CE}
			END;
		15:BEGIN                                        {256K}
				PCEB_PIN26(FALSE);				{A9}
				PCEB_PIN24(FALSE);				{OE}
				PCEB_PIN30(FALSE);				{Vcc}
				PCEB_PIN3(TRUE);					{Vpp}
				PCEB_PIN31(FALSE);				{DON'T CARE}
				PCEB_PIN29(FALSE);				{A14}
				PCEB_PIN22(FALSE);				{CE}
			END;
		16:BEGIN                                        {512K}
				PCEB_PIN26(FALSE);				{A9}
				PCEB_PIN24(FALSE);				{OE}
				PCEB_PIN30(FALSE);				{Vcc}
				PCEB_PIN3(FALSE);					{A15}
				PCEB_PIN31(FALSE);				{DON'T CARE}
				PCEB_PIN29(FALSE);				{A14}
				PCEB_PIN22(FALSE);				{CE}
			END;
		17:BEGIN                                        {1M}
				PCEB_PIN26(FALSE);				{A9}
				PCEB_PIN24(FALSE);				{OE}
				PCEB_PIN30(TRUE);					{DON'T CARE, BUT A17 IS BETTER}
				PCEB_PIN3(FALSE);					{A15}
				PCEB_PIN31(TRUE);					{PGM}
				PCEB_PIN29(FALSE);				{A14}
				PCEB_PIN22(FALSE);				{CE}
			END;
		18:BEGIN                                        {2M}
				PCEB_PIN26(FALSE);				{A9}
				PCEB_PIN24(FALSE);				{OE}
				PCEB_PIN30(TRUE);					{A17}
				PCEB_PIN3(FALSE);					{A15}
				PCEB_PIN31(TRUE);					{PGM}
				PCEB_PIN29(FALSE);				{A14}
				PCEB_PIN22(FALSE);				{CE}
			END;
		19:BEGIN                                        {4M}
				PCEB_PIN26(FALSE);				{A9}
				PCEB_PIN24(FALSE);				{OE}
				PCEB_PIN30(TRUE);					{A17}
				PCEB_PIN3(FALSE);					{A15}
				PCEB_PIN31(FALSE);				{A18}
				PCEB_PIN29(FALSE);				{A14}
				PCEB_PIN22(FALSE);				{CE}
			END;
	END;
END;


{**********************************************************************
SETS THE PIN STATES REQUIRED BY THE CURRENTLY SELECTED DEVICE TYPE
FOR PROGRAMMING MODE ONLY
**********************************************************************}
PROCEDURE SET_PROG_PIN_STATE;
BEGIN
	CASE ESIZE OF
		13:BEGIN														{64K}
				PCEB_PIN26(FALSE);				{A9}
				PCEB_PIN24(FALSE);				{OE}
				PCEB_PIN30(FALSE);				{Vcc}
				PCEB_PIN3(TRUE);					{Vpp}
				PCEB_PIN31(FALSE);				{DON'T CARE}
				PCEB_PIN29(TRUE);					{PGM}
				PCEB_PIN22(FALSE);				{CE}
			END;
		14:BEGIN                                        {128K}
				PCEB_PIN26(FALSE);				{A9}
				PCEB_PIN24(FALSE);				{OE}
				PCEB_PIN30(FALSE);				{Vcc}
				PCEB_PIN3(TRUE);					{Vpp}
				PCEB_PIN31(FALSE);				{DON'T CARE}
				PCEB_PIN29(TRUE);					{PGM}
				PCEB_PIN22(FALSE);				{CE}
			END;
		15:BEGIN                                        {256K}
				PCEB_PIN26(FALSE);				{A9}
				PCEB_PIN24(FALSE);				{OE}
				PCEB_PIN30(FALSE);				{Vcc}
				PCEB_PIN3(TRUE);					{Vpp}
				PCEB_PIN31(FALSE);				{DON'T CARE}
				PCEB_PIN29(FALSE);				{A14}
				PCEB_PIN22(TRUE);					{PGM}
			END;
		16:BEGIN                                        {512K}
				PCEB_PIN26(FALSE);				{A9}
				PCEB_PIN24(TRUE);					{VPP}
				PCEB_PIN30(FALSE);				{Vcc}
				PCEB_PIN3(FALSE);					{A15}
				PCEB_PIN31(FALSE);				{DON'T CARE}
				PCEB_PIN29(FALSE);				{A14}
				PCEB_PIN22(TRUE);					{PGM}
			END;
		17:BEGIN                                        {1M}
				PCEB_PIN26(FALSE);				{A9}
				PCEB_PIN24(FALSE);				{OE}
				PCEB_PIN30(TRUE);					{DON'T CARE, BUT A17 IS BETTER}
				PCEB_PIN3(FALSE);					{A15}
				PCEB_PIN31(TRUE);					{PGM}
				PCEB_PIN29(FALSE);				{A14}
				PCEB_PIN22(FALSE);				{CE}
			END;
		18:BEGIN                                        {2M}
				PCEB_PIN26(FALSE);				{A9}
				PCEB_PIN24(FALSE);				{OE}
				PCEB_PIN30(TRUE);					{A17}
				PCEB_PIN3(FALSE);					{A15}
				PCEB_PIN31(TRUE);					{PGM}
				PCEB_PIN29(FALSE);				{A14}
				PCEB_PIN22(FALSE);				{CE}
			END;
		19:BEGIN                                        {4M}
				PCEB_PIN26(FALSE);				{A9}
				PCEB_PIN24(FALSE);				{OE}
				PCEB_PIN30(TRUE);					{A17}
				PCEB_PIN3(FALSE);					{A15}
				PCEB_PIN31(FALSE);				{A18}
				PCEB_PIN29(FALSE);				{A14}
				PCEB_PIN22(TRUE);					{PGM}
			END;
	END;
END;


{******************************************************}
PROCEDURE SELECT_ID;
BEGIN
END;


{******************************************************}
FUNCTION BLANK_CHECK:BOOLEAN;
VAR C:LONGINT;
	B:BYTE;
BEGIN
	BLANK_CHECK:=FALSE;
	PCEB_STABLE;
	PCEB_VCC(TRUE);
	PCEB_OE(FALSE);										{SET OE LOW}
	PCEB_CE(FALSE);                              {SET CE LOW}
	SET_READ_PIN_STATE;									{SET ALL PINS}
	LASTADDR:=0;
	DISPLAY_SETTINGS;
	DISPLAY_MESSAGE('Checking if device is blank. Please wait...');
	DISPCOUNT:=0;                       	      {ADDR DISPLAY COUNTER}

	FOR C:=0 TO ENDADDR DO BEGIN
		DISPLAY_ADDRESS(C);
		IF C>=STARTADDR THEN BEGIN
			IF (PCEB_GETBYTE <> $FF) THEN BEGIN 	  	{READ BYTE}
				LASTMESSAGE:='Device FAILED Blank check.';
				LASTADDR:=C;
				DISPLAY_MESSAGE('');
				BEEP;
				EXIT;
			END;
			IF KEYPRESSED THEN
				IF READKEY=#27 THEN BEGIN
					LASTMESSAGE:='Device is blank up to Last Address.';
					LASTADDR:=C;
					DISPLAY_MESSAGE('');
					BEEP;
					EXIT;
				END;
		END;
		PCEB_ACLK;					              		{INCREMENT ADDRESS}
	END;

	BLANK_CHECK:=TRUE;
	LASTMESSAGE:='Device is BLANK';
	LASTADDR:=C;
	DISPLAY_MESSAGE('');
	BEEP;
END;


{******************************************************}
PROCEDURE READ_DEVICE;
VAR C,A,B:LONGINT;
	CH:CHAR;
BEGIN
	BEEP;
	DISPLAY_MESSAGE('Read the device using current options, Y/N ?');
	CH:=UPCASE(READKEY);
	LASTADDR:=0;
	DISPLAY_SETTINGS;
	IF CH<>'Y' THEN EXIT;
	IF NOT DEVICE_LOADED THEN EXIT;
	CLEAR_ARRAY;                        			{CLEAR THE BUFFER}
	C:=0;
	B:=0;                               			{BUFFER BLOCK NUMBER}
	DISPCOUNT:=0;                       			{ADDR DISPLAY COUNTER}

	PCEB_STABLE;
	SET_READ_PIN_STATE;									{SET ALL PINS}
	PCEB_VCC(TRUE);
	PCEB_OE(FALSE);										{SET OE LOW}
	PCEB_CE(FALSE);                              {SET CE LOW}
	A:=0;														{BLOCK OFFSET ADDRESS COUNTER}
	DISPLAY_MESSAGE('Reading device contents. Please wait...');

	FOR C:=0 TO ENDADDR DO BEGIN        			{ABSOLUTE ADDRESS COUNTER}
		DISPLAY_ADDRESS(C);
		IF C>=STARTADDR THEN BEGIN
			DATA[B]^.D[A]:=PCEB_GETBYTE;  	 			{READ THE BYTE}

			IF KEYPRESSED THEN
				IF READKEY=#27 THEN BEGIN
					DISPLAY_MESSAGE('');
					LASTMESSAGE:='Read operation stopped at Last Address.';
					LASTADDR:=C;
					BEEP;
					EXIT;
				END;

			INC(A);									{INCREMENT BLOCK OFFSET COUNTER}

			IF A>MAXD THEN BEGIN             {IF END OF BLOCK}
				A:=0;                         {RESET BLOCK OFFSET COUNTER}
				INC(B);                       {INCREMENT BLOCK COUNTER}
			END;
		END;

		PCEB_ACLK;     						{INCREMENT ADDRESS}
	END;

	PCEB_STABLE;								{STABLIZE BEFORE LENGTHY CHECKSUM}
	LASTADDR:=C;
	LASTMESSAGE:='Device contents read OK.';
	DISPLAY_MESSAGE('');
	BEEP;
END;

{******************************************************}
FUNCTION COMPARE_DEVICE(VCC6V:BOOLEAN):BOOLEAN;
VAR C,A,B : LONGINT;
	FAIL:BOOLEAN;
BEGIN
	COMPARE_DEVICE:=FALSE;
	IF NOT DEVICE_LOADED THEN EXIT;
	FAIL:=FALSE;
	C:=0;
	B:=0;
	DISPCOUNT:=0;                             {ADDR DISPLAY COUNTER}
	PCEB_STABLE;
	IF VCC6V THEN PCEB_SUPPLY(TRUE);				{6V SUPPLY VOLTAGE}
	PCEB_VCC(TRUE);
	PCEB_OE(FALSE);									{SET OE LOW}
	PCEB_CE(FALSE);									{SET CE LOW}
	SET_READ_PIN_STATE;								{SET ALL PINS}
	LASTADDR:=0;
	DISPLAY_SETTINGS;
	A:=0;													{BLOCK OFFSET ADDRESS COUNTER}

	IF VCC6V THEN DISPLAY_MESSAGE('Verifing Device. Please wait...')
		ELSE DISPLAY_MESSAGE('Comparing Device. Please wait...');

	FOR C:=0 TO ENDADDR DO BEGIN            	{ABSOLUTE ADDRESS COUNTER}
		DISPLAY_ADDRESS(C);
		IF C>=STARTADDR THEN BEGIN
			IF PCEB_GETBYTE <> DATA[B]^.D[A] THEN FAIL:=TRUE;   {READ BYTE}

			IF FAIL THEN BEGIN                  {THE COMPARE TEST HAS FAILED}
				LASTMESSAGE:='Device does NOT compare with memory.';
				LASTADDR:=C;
				DISPLAY_MESSAGE('');
				BEEP;
				EXIT;
			END;
			IF KEYPRESSED THEN
				IF READKEY=#27 THEN BEGIN
					LASTMESSAGE:='Device compares with memory up to Last Address.';
					LASTADDR:=C;
					DISPLAY_MESSAGE('');
					BEEP;
					EXIT;
				END;

			INC(A);											{INCREMENT BLOCK OFFSET COUNTER}

			IF A>MAXD THEN BEGIN							{IF END OF BLOCK}
				A:=0;                               {RESET BLOCK OFFSET COUNTER}
				INC(B);										{INCREMENT BLOCK NUMBER}
			END;
		END;

		PCEB_ACLK;			                		{INCREMENT ADDRESS}
	END;

	PCEB_STABLE;
	LASTADDR:=C;
	COMPARE_DEVICE:=TRUE;
	LASTMESSAGE:='Device compares with memory OK.';
	DISPLAY_MESSAGE('');
	BEEP;
END;

{******************************************************
PROGRAMS A BYTE INTO THE CURRENT DEVICE'S ADDRESS.
******************************************************}
FUNCTION PROGRAM_BYTE(B:BYTE) : BOOLEAN;
VAR R,READBACK,D:BYTE;
	C:LONGINT;
BEGIN
	PROGRAM_BYTE:=TRUE;
	PCEB_PGM(TRUE);										{DISABLE PROGRAMMING}

	R:=1;                                  		{PROGRAM RETRY COUNTER}
	PCEB_SETBYTE(B);
	REPEAT
		CASE ALG OF
			0:	BEGIN						{STANDARD GENERIC EPROM ALGORITHM (NOT 27512)}
				PCEB_VPP(TRUE);									{TURN VPP ON}
				PCEB_OE(TRUE);										{DIABLE PROG.}
				PCEB_CE(FALSE);                          	{ENABLE CHIP}
				DELAY5;												{SETUP DELAY TIME}
				PCEB_PGM(FALSE);                          	{START PROG. PULSE}
				IF PULSE>0 THEN DELAY(PULSE) ELSE DELAY100;	{PROG. PULSE DELAY}
				PCEB_PGM(TRUE);
				DELAY5;     										{HOLD TIME}
				PCEB_OE(FALSE);								{ALLOW DATA VERIFY}
				END;
		END;
		IF PCEB_GETBYTE=B THEN EXIT;						{VERIFIED OK}
		INC(R);
	UNTIL R>RETRY;  	       								{RETRY IF FAILED}
	PROGRAM_BYTE:=FALSE; 	                        {BYTE HAS FAILED}
END;

{******************************************************}
FUNCTION PROGRAM_DEVICE(SKIP:BOOLEAN):BOOLEAN;
VAR C,A,B:LONGINT;
	CH:CHAR;
	NONSTOP:BOOLEAN;
	ERR:BOOLEAN;
BEGIN
	IF NOT SKIP THEN BEGIN
		BEEP;
		LASTMESSAGE:='Program the device using current options, Y/N ?';
		DISPLAY_SETTINGS;
		CH:=UPCASE(READKEY);
		LASTMESSAGE:='';
		DISPLAY_SETTINGS;
		IF CH<>'Y' THEN EXIT;
	END;
	PROGRAM_DEVICE:=FALSE;
	IF NOT DEVICE_LOADED THEN EXIT;
	C:=0;                                     {BUFFER COUNTER}
	B:=0;                                     {BUFFER BLOCK COUNTER}
	DISPCOUNT:=0;                             {ADDR DISPLAY COUNTER}
	LASTADDR:=0;
	DISPLAY_SETTINGS;
	PCEB_STABLE;
	IF VCC=6 THEN PCEB_SUPPLY(TRUE);				{6V SUPPLY IF SET}
	PCEB_VCC(TRUE);									{TURN ON VCC SUPPLY}
	PCEB_OE(FALSE);									{SET OE LOW}
	PCEB_CE(FALSE);                           {SET CE LOW}
	SET_PROG_PIN_STATE;								{SET ALL PINS}
	A:=0; 			                           {BLOCK OFFSET ADDRESS COUNTER}
	NONSTOP:=FALSE;                           {INITIALLY STOP ON ERROR}
	ERR:=FALSE;											{ASSUME ERROR FALSE}
	DISPLAY_MESSAGE('Programming Device. Please Wait...');

	FOR C:=0 TO ENDADDR DO BEGIN
		DISPLAY_ADDRESS(C);
		IF C>=STARTADDR THEN BEGIN
			IF NOT PROGRAM_BYTE(DATA[B]^.D[A]) THEN BEGIN      {PROGRAM BYTE}

				IF NOT NONSTOP THEN BEGIN
					ERR:=TRUE;
					BEEP;
					LASTMESSAGE:='Program byte failed. (C)ontinue, (N)onstop, or (S)top ?';
					DISPLAY_MESSAGE('');
					DISPLAY_SETTINGS;
					CH:=UPCASE(READKEY);
					IF CH='S' THEN BEGIN
						SET_READ_PIN_STATE;
						PCEB_STABLE;
						LASTADDR:=C;
						LASTMESSAGE:='Device FAILED to program correctly.';
						DISPLAY_MESSAGE('');
						EXIT;
					END;
					DISPLAY_MESSAGE('Programming Device. Please Wait...');
					IF CH='N' THEN NONSTOP:=TRUE;
				END;
			END;
			IF KEYPRESSED THEN
				IF READKEY=#27 THEN BEGIN
					IF ERR THEN LASTMESSAGE:='Programming STOPPED at Last Address. An ERROR also occured.'
						ELSE LASTMESSAGE:='Device programmed correctly up to Last Address.';
					SET_READ_PIN_STATE;
					PCEB_STABLE;
					LASTADDR:=C;
					DISPLAY_MESSAGE('');
					EXIT;
				END;

			INC(A);											{INCREMENT OFFSET COUNTER}

			IF A>MAXD THEN BEGIN							{IF END OF BLOCK}
				A:=0;                               {RESET BLOCK OFFSET COUNTER}
				INC(B);										{INCREMENT BLOCK NUMBER}
			END;
		END;

		PCEB_ACLK;										{INCREMENT ADDRESS}
	END;

	SET_READ_PIN_STATE;
	PCEB_STABLE;
	PROGRAM_DEVICE:=TRUE;
	LASTMESSAGE:='Device has been programmed OK.';
	LASTADDR:=C;
	DISPLAY_MESSAGE('');
	BEEP;
END;

{******************************************************}
PROCEDURE AUTO_PROGRAM;
BEGIN
   IF NOT DEVICE_LOADED THEN EXIT;
   IF NOT BLANK_CHECK THEN EXIT;
	DISPLAY_SETTINGS;
	IF NOT PROGRAM_DEVICE(TRUE) THEN EXIT;
   DISPLAY_SETTINGS;
	IF NOT COMPARE_DEVICE(FALSE) THEN              {AT VCC=5V}
      LASTMESSAGE:='Device Blank checked, Programmed and Compared OK';
END;

{******************************************************}
PROCEDURE GET_STARTADDR;
VAR V:LONGINT;
	CH:CHAR;
	S:STRING;
BEGIN
	COLOR(FC2,BC2);
	WRITEXY(48,18,'        ');
	S:='';
	CH:=' ';
	REPEAT
		WRITEXY(48,18,S+' ');
		CH:=UPCASE(READKEY);
		CASE CH OF
			'A'..'F', '0'..'9': IF LENGTH(S)<6 THEN S:=S+CH;
			#8: S:=COPY(S,1,LENGTH(S)-1);
		END;
	UNTIL (CH=#13) OR (CH=#27);

	V:=HEX_TO_DEC(S);
	IF CH=#13 THEN BEGIN
		IF (V>=SIZE) OR (V>=ENDADDR) THEN BEGIN
			BEEP;
			DISPLAY_MESSAGE('Out of range. Press any key.');
			CH:=READKEY;
			DISPLAY_MESSAGE('');
			END
		ELSE BEGIN
			STARTADDR:=V;
			CALCULATE_CRC;
		END;
	END;
END;

{******************************************************}
PROCEDURE GET_ENDADDR;
VAR V:LONGINT;
	CH:CHAR;
	S:STRING;
BEGIN
	COLOR(FC2,BC2);
	WRITEXY(48,19,'        ');
	S:='';
	CH:=' ';
	REPEAT
		WRITEXY(48,19,S+' ');
		CH:=UPCASE(READKEY);
		CASE CH OF
			'A'..'F', '0'..'9': IF LENGTH(S)<6 THEN S:=S+CH;
			#8: S:=COPY(S,1,LENGTH(S)-1);
		END;
	UNTIL (CH=#13) OR (CH=#27);

	V:=HEX_TO_DEC(S);
	IF CH=#13 THEN BEGIN
		IF (V>=SIZE) OR (V<=STARTADDR) THEN BEGIN
			BEEP;
			DISPLAY_MESSAGE('Out of range. Press any key.');
			CH:=READKEY;
			DISPLAY_MESSAGE('');
			END
		ELSE BEGIN
			ENDADDR:=V;
			CALCULATE_CRC;								{NEED TO RECALC CRC}
		END;
	END;
END;


{******************************************************}
PROCEDURE CHECK_MEMORY;
BEGIN
	IF MEMAVAIL<525000 THEN BEGIN
		CLRSCR;
		BEEP;
		WRITELN('You must have at least 590KB of base memory free to support 4Mbit devices.');
		WRITELN('Device sizes up to 2Mbit only will be supported.');
		WRITELN('Do not attempt to use the 4Mbit option, doing so may cause unexpected results.');
		WRITELN;
		WRITELN('Press any key to continue');
		WHILE NOT KEYPRESSED DO;
		NUMBLOCKS:=31;
	END;
END;


{******************************************************}
PROCEDURE MAIN_MENU;
VAR
	CH:CHAR;
	C,D:BYTE;
	DUMMY:BOOLEAN;
BEGIN
C:=1;
REPEAT
	COLOR(H1,BC);
	GOTOXY(1,1);
	IF C>0 THEN CLRSCR;
	WRITELN(' PC-EPROM Burner - Version 2.00 - (c)1995 David L. Jones Advanced R&D Solutions');
	WRITELN(REPSTR('',80));
	TEXTCOLOR(H2);

	WRITEXY     (2,3,''+REPSTR('',23)+'');
	FOR D:=0 TO 16 DO BEGIN
		WRITEXY(2,4+D,'');
		WRITEXY(26,4+D,'');
	END;
	WRITEXY    (2,21,''+REPSTR('',23)+'');

	WRITEXY     (27,3,''+REPSTR('',51)+'');
	FOR D:=0 TO 16 DO BEGIN
		WRITEXY(27,4+D,'');
		WRITEXY(79,4+D,'');
	END;
	WRITEXY    (27,21,''+REPSTR('',51)+'');

	WRITEXY(10,3,'Options');
	WRITEXY(50,3,'Settings');

	TEXTCOLOR(FC);
	HIGHVIDEO; WRITEXY(5,4, 'A'); LOWVIDEO; WRITE('uto program');
	HIGHVIDEO; WRITEXY(5,5, 'P'); LOWVIDEO; WRITE('rogram');
	HIGHVIDEO; WRITEXY(5,6, 'B'); LOWVIDEO; WRITE('lank check');
	HIGHVIDEO; WRITEXY(5,7, 'R'); LOWVIDEO; WRITE('ead');
	HIGHVIDEO; WRITEXY(5,8, 'V'); LOWVIDEO; WRITE('erify  (6V)');
	HIGHVIDEO; WRITEXY(5,9, 'C'); LOWVIDEO; WRITE('ompare (5V)');
	HIGHVIDEO; WRITEXY(5,10,'L'); LOWVIDEO; WRITE('oad data');
	HIGHVIDEO; WRITEXY(5,11,'S'); LOWVIDEO; WRITE('ave data');
	HIGHVIDEO; WRITEXY(5,12,'E'); LOWVIDEO; WRITE('dit data');
	HIGHVIDEO; WRITEXY(5,13,'D'); LOWVIDEO; WRITE('isplay data');
	HIGHVIDEO; WRITEXY(5,14,'F'); LOWVIDEO; WRITE('ilename');
	HIGHVIDEO; WRITEXY(5,15,'\'); LOWVIDEO; WRITE(' Directory');
	HIGHVIDEO; WRITEXY(5,16,'T'); LOWVIDEO; WRITE('ype Library');
	HIGHVIDEO; WRITEXY(5,17,'I'); LOWVIDEO; WRITE('D Check : ');
	HIGHVIDEO; WRITEXY(5,18,'[]');LOWVIDEO; WRITE(' Start/End Address');
	HIGHVIDEO; WRITEXY(5,19,'H'); LOWVIDEO; WRITE('ex Address : ');
	HIGHVIDEO; WRITEXY(5,20,'Q'); LOWVIDEO; WRITE('uit');

	DISPLAY_SETTINGS;
	C:=0;

	SET_READ_PIN_STATE;

	REPEAT
		PCEB_STABLE;      		                  {KEEP THE UNIT STABLE}
	UNTIL KEYPRESSED;

	CH:=UPCASE(READKEY);
	CASE CH OF
		#0: CH:=READKEY;                          {TRAP EXTENDED KEYPRESS}
		'A': AUTO_PROGRAM;
		'E': IF DEVICE_LOADED THEN BEGIN
			EDITOR(TRUE);                          {EDIT THE TEMPFILE}
			C:=1;
			CALCULATE_CRC;									{RECALCULATE THE CHECKSUM}
			END;
		'D': IF DEVICE_LOADED THEN BEGIN
			EDITOR(FALSE);                         {VIEW THE TEMPFILE}
			C:=1;
			END;
		'F': FILENAME:=GET_FILENAME;
		'L': BEGIN
				DUMMY:=LOAD_FILE(TRUE);
				CALCULATE_CRC;								{RECALCULATE THE CHECKSUM}
			END;
		'S': DUMMY:=SAVE_FILE;
		'T': BEGIN
			TYPE_LIBRARY;
			C:=1;
			CALCULATE_CRC;									{RECALCULATE THE CHECKSUM}
			END;
		'H': IF BASEADDR=$378 THEN BASEADDR:=$278
				ELSE IF BASEADDR=$278 THEN BASEADDR:=$3BC
						ELSE IF BASEADDR=$3BC THEN BASEADDR:=$378
							ELSE BASEADDR:=$278;
		'B': DUMMY:=BLANK_CHECK;
		'C': DUMMY:=COMPARE_DEVICE(FALSE);                        {VCC=5V}
		'V': DUMMY:=COMPARE_DEVICE(TRUE);                         {VCC=6V}
		'R': BEGIN
				READ_DEVICE;
				CALCULATE_CRC;								{RECALCULATE THE CHECKSUM}
			END;
		'P': DUMMY:=PROGRAM_DEVICE(FALSE);
		'\': BEGIN DISPLAY_FILES; C:=1; END;
		'[': GET_STARTADDR;
		']': GET_ENDADDR;
		'I': AUTOID:=NOT AUTOID;
		'Q': EXIT;
	ELSE
		C:=0;
	END;
UNTIL CH='Q';
END;

{*************************************************}
{ MAIN PROGRAM ROUTINE}
{*************************************************}
BEGIN
	NUMBLOCKS:=63;							{64X8192=512KB=4Mbit buffer}
	CHECK_MEMORY;
	INITIALISE_DATA_ARRAY;
	CLEAR_ARRAY;
	CHECKBREAK:=FALSE;					{TURN OFF CTRL-BREAK CHECKING}
	TEXTMODE(CO80);						{RESET TEXT MODE JUST IN CASE}
	REG.CH:=32;                      {MAGIC VALUE TO TURN OFF THE CURSOR}
	REG.AH:=1;                       {"SET CURSOR SIZE" DOS FUNCTION}
	INTR($10,REG);                   {CALL INTERRUPT 10H}
	FC:=WHITE; FC2:=WHITE;
	BC:=LIGHTBLUE;  BC2:=LIGHTGRAY;
	H1:=LIGHTCYAN; H2:=YELLOW;
	DLY:=50;                         {DEFAULT DELAY VALUE FOR DUMMY LOOP}
	IF PARAMSTR(1)='M' THEN BEGIN		{IF MONOCHROME}
		FC:=LIGHTGRAY; FC2:=BLACK;
		BC:=BLACK; BC2:=LIGHTGRAY;
		H1:=LIGHTGRAY; H2:=LIGHTGRAY;
	END;
	IF PARAMSTR(2)<>'' THEN BEGIN		{LOAD COMMAND LINE DELAY VARIABLE}
		VAL(PARAMSTR(2), DLY, DLY);
		IF DLY<1 THEN DLY:=1;
	END;
	COLOR(FC,BC);
	CLRSCR;
	LOAD_EPROM_TYPES;
	FILENAME:='';
	LASTMESSAGE:='';
	ETYPE:=0;
	SIZE:=1;
	STARTADDR:=0;
	ENDADDR:=SIZE-1;
	BASEADDR:=$378;
	LOAD_SETUP;
	CALIBRATE_TIMER;
	CALCULATE_CRC;
	MAIN_MENU;
	SAVE_SETUP;
	REG.CH:=11;                      {RESIZE THE CURSOR}
	REG.CL:=12;
	REG.AH:=1;                       {"SET CURSOR SIZE" DOS FUNCTION}
	INTR($10,REG);                   {CALL INTERRUPT 10H}
	NORMVIDEO;
	COLOR(7,0);
	CLRSCR;
END.
