{ 	PC-Eprom Burner  Interface Library
	Revision 1.0
	Started 1.0 - 2/10/95
	Last Edited - 8/10/95
	Copyright (C) 1995 David L. Jones
	Advanced R&D Solutions
}

UNIT PCEBIL10;

{**************************************************************}
INTERFACE

USES CRT, DOS;
VAR
	BASEADDR:WORD;							{PARALLEL PORT BASE ADDRESS}
	REG1, REG2 : BYTE; 					{DATA REGISTERS ON PCEB}

FUNCTION  PCEB_GETBYTE:BYTE;			{GET CURRENT BYTE ON PCEB DATA BUS}
PROCEDURE PCEB_STABLE;           	{PUT PCEB INTO A STABLE STATE}
PROCEDURE PCEB_OE(STATE:BOOLEAN);   {SET OE LINE ON PCEB}
													{TRUE=OE HIGH, FALSE=OE LOW}
PROCEDURE PCEB_CE(STATE:BOOLEAN);	{SET CE LINE ON PCEB}
													{TRUE=CE HIGH, FALSE=CE LOW}
PROCEDURE PCEB_PGM(STATE:BOOLEAN);	{SET PGM LINE ON PCEB}
													{TRUE=PGM HIGH, FALSE=PGM LOW}
PROCEDURE PCEB_ACLK;						{CLOCK THE ACLK HIGH-LOW ON PCEB}
													{TOGGLES ACLK LINE HIGH-LOW}
PROCEDURE PCEB_ARST;						{RESET THE ADDRESS COUNTER ON PCEB}
													{PULSES RESET LINE}
PROCEDURE PCEB_VPP(STATE:BOOLEAN);	{TURN Vpp ON/OFF ON PCEB}
													{TRUE=Vpp ON, FALSE=Vpp OFF}
PROCEDURE PCEB_SUPPLY(STATE:BOOLEAN);	{SET SUPPLY VOLTAGE FOR EPROM}
														{TRUE=6V, FALSE=5V}
FUNCTION  PCEB_ID_MANUF:BYTE;			{GET MANUFACTURER ID CODE}
FUNCTION  PCEB_ID_DEVICE:BYTE;		{GET DEVICE ID CODE}
PROCEDURE PCEB_SETBYTE(NUM:BYTE);	{LATCH BYTE ONTO DATA BUS REGISTER}
PROCEDURE PCEB_PIN26(STATE:BOOLEAN);{SET PIN 26 RELAY}
													{TRUE=Vpp, FALSE=A9}
PROCEDURE PCEB_PIN24(STATE:BOOLEAN);{SET PIN 24 RELAY}
													{TRUE=Vpp, FALSE=OE}
PROCEDURE PCEB_PIN30(STATE:BOOLEAN);{SET PIN 30 RELAY}
													{TRUE=A17, FALSE=Vcc}
PROCEDURE PCEB_PIN3(STATE:BOOLEAN);	{SET PIN 3 RELAY}
													{TRUE=Vpp, FALSE=A15}
PROCEDURE PCEB_PIN31(STATE:BOOLEAN);{SET PIN 31 MUX}
													{TRUE=PGM, FALSE=A18}
PROCEDURE PCEB_PIN29(STATE:BOOLEAN);{SET PIN 29 MUX}
													{TRUE=PGM, FALSE=A14}
PROCEDURE PCEB_PIN22(STATE:BOOLEAN);{SET PIN 22 MUX}
													{TRUE=PGM, FALSE=CE}
PROCEDURE PCEB_VCC(STATE:BOOLEAN);	{TURN VCC POWER TO EPROM ON & OFF}
													{TRUE=Vcc ON, FALSE=Vcc OFF}

{***************************************************************}
IMPLEMENTATION

PROCEDURE PCEB_LATCH1;					{TOGGLE LATCH1}
BEGIN
											{ALLOW FOR INVERTED BIT IN PORT}
	PORT[BASEADDR+2]:=PORT[BASEADDR+2] OR 2;			{SET LTCH1 LOW}
	PORT[BASEADDR+2]:=PORT[BASEADDR+2] AND $FD;		{SET LTCH1 HIGH}
	PORT[BASEADDR+2]:=PORT[BASEADDR+2] OR 2;			{SET LTCH1 LOW}
END;

{*****************************************************************}
PROCEDURE PCEB_LATCH2;					{TOGGLE LATCH2}
BEGIN
											{ALLOW FOR INVERTED BIT IN PORT}
	PORT[BASEADDR+2]:=PORT[BASEADDR+2] OR 1;			{SET LTCH2 LOW}
	PORT[BASEADDR+2]:=PORT[BASEADDR+2] AND $FE;		{SET LTCH2 HIGH}
	PORT[BASEADDR+2]:=PORT[BASEADDR+2] OR 1;			{SET LTCH2 LOW}
END;

{*****************************************************************}
PROCEDURE PCEB_LATCH3;					{TOGGLE LATCH3}
BEGIN
	PORT[BASEADDR+2]:=PORT[BASEADDR+2] AND $FB;		{SET LTCH3 LOW}
	PORT[BASEADDR+2]:=PORT[BASEADDR+2] OR 4;			{SET LTCH3 HIGH}
	PORT[BASEADDR+2]:=PORT[BASEADDR+2] AND $FB;		{SET LTCH3 LOW}
END;

{*****************************************************************}
FUNCTION  PCEB_GETBYTE:BYTE;			{GET CURRENT BYTE ON PCEB DATA BUS}
VAR NIBBLE,VALUE,D:BYTE;
BEGIN
	VALUE:=0;
	REG1:=REG1 OR 4;										{SET SNIB LOW}
	PORT[BASEADDR]:=REG1;
	PCEB_LATCH1;
	NIBBLE:=PORT[BASEADDR+1] XOR $80;					{GET BYTE & INVERT LAST BIT}
	IF (NIBBLE AND $80)>0 THEN VALUE:=VALUE+64;
	IF (NIBBLE AND $40)>0 THEN VALUE:=VALUE+128;
	IF (NIBBLE AND $20)>0 THEN VALUE:=VALUE+32;
	IF (NIBBLE AND $10)>0 THEN VALUE:=VALUE+16;
	REG1:=REG1 AND $FB;											{SET SNIB HIGH}
	PORT[BASEADDR]:=REG1;
	PCEB_LATCH1;
	NIBBLE:=PORT[BASEADDR+1] XOR $80;					{GET BYTE & INVERT LAST BIT}
	IF (NIBBLE AND $80)>0 THEN VALUE:=VALUE+4;
	IF (NIBBLE AND $40)>0 THEN VALUE:=VALUE+8;
	IF (NIBBLE AND $20)>0 THEN VALUE:=VALUE+2;
	IF (NIBBLE AND $10)>0 THEN VALUE:=VALUE+1;
	PCEB_GETBYTE:=VALUE;

END;

{*****************************************************************}
PROCEDURE PCEB_OE(STATE:BOOLEAN);   {SET OE LINE ON PCEB}
													{TRUE=OE HIGH, FALSE=OE LOW}
BEGIN
	IF STATE THEN REG2:=REG2 OR 16	{SET OE HIGH}
		ELSE REG2:=REG2 AND $EF;		{SET OE LOW}
	PORT[BASEADDR]:=REG2;
	PCEB_LATCH2;
END;

{*****************************************************************}
PROCEDURE PCEB_CE(STATE:BOOLEAN);	{SET CE LINE ON PCEB}
													{TRUE=CE HIGH, FALSE=CE LOW}
BEGIN
	IF STATE THEN REG1:=REG1 OR 8		{SET CE HIGH}
		ELSE REG1:=REG1 AND $F7;		{SET CE LOW}
	PORT[BASEADDR]:=REG1;
	PCEB_LATCH1;
END;

{*****************************************************************}
PROCEDURE PCEB_PGM(STATE:BOOLEAN);	{SET PGM LINE ON PCEB}
													{TRUE=PGM HIGH, FALSE=PGM LOW}
BEGIN
	IF STATE THEN REG1:=REG1 OR 1		{SET PGM HIGH}
		ELSE REG1:=REG1 AND $FE;		{SET PGM LOW}
	PORT[BASEADDR]:=REG1;
	PCEB_LATCH1;
END;

{*****************************************************************}
PROCEDURE PCEB_ACLK;						{CLOCK THE ACLK HIGH-LOW ON PCEB}
													{TOGGLES ACLK LINE HIGH-LOW}
BEGIN
	REG1:=REG1 AND $DF;					{SET ACLK LOW}
	PORT[BASEADDR]:=REG1;
	PCEB_LATCH1;

	REG1:=REG1 OR 32;						{SET ACLK HIGH}
	PORT[BASEADDR]:=REG1;
	PCEB_LATCH1;

	REG1:=REG1 AND $DF;					{SET ACLK LOW}
	PORT[BASEADDR]:=REG1;
	PCEB_LATCH1;
END;

{*****************************************************************}
PROCEDURE PCEB_ARST;						{RESET THE ADDRESS COUNTER ON PCEB}
													{PULSES RESET LINE}
BEGIN
	REG1:=REG1 OR 16;						{SET ARST HIGH}
	PORT[BASEADDR]:=REG1;
	PCEB_LATCH1;

	REG1:=REG1 AND $EF;					{SET ARST LOW}
	PORT[BASEADDR]:=REG1;
	PCEB_LATCH1;
END;

{*****************************************************************}
PROCEDURE PCEB_VPP(STATE:BOOLEAN);	{TURN Vpp ON/OFF ON PCEB}
													{TRUE=Vpp ON, FALSE=Vpp OFF}
BEGIN
	IF STATE THEN REG1:=REG1 OR 64	{SET VPP HIGH}
		ELSE REG1:=REG1 AND $BF;		{SET VPP LOW}
	PORT[BASEADDR]:=REG1;
	PCEB_LATCH1;
END;

{*****************************************************************}
PROCEDURE PCEB_SUPPLY(STATE:BOOLEAN);	{SET SUPPLY VOLTAGE FOR EPROM}
														{TRUE=6V, FALSE=5V}
BEGIN
												{NOTE: MUST ALLOW FOR TRANI INVERTER}
	IF STATE THEN REG1:=(REG1 AND $7F)		{SET SUPPLY LOW}
		ELSE REG1:=(REG1 OR 128);				{SET SUPPLY HIGH}
	PORT[BASEADDR]:=REG1;
	PCEB_LATCH1;
END;

{*****************************************************************}
PROCEDURE PCEB_SETBYTE(NUM:BYTE);	{LATCH BYTE ONTO DATA BUS REGISTER}
BEGIN
	PORT[BASEADDR]:=NUM;
	PCEB_LATCH3;
END;

{*****************************************************************}
PROCEDURE PCEB_PIN26(STATE:BOOLEAN);{SET PIN 26 RELAY}
													{TRUE=Vpp, FALSE=A9}
BEGIN
	IF STATE THEN REG2:=REG2 OR 1		{SET VPP}
		ELSE REG2:=REG2 AND $FE;		{SET A9}
	PORT[BASEADDR]:=REG2;
	PCEB_LATCH2;
	DELAY(50);
END;

{*****************************************************************}
PROCEDURE PCEB_PIN24(STATE:BOOLEAN);{SET PIN 24 RELAY}
													{TRUE=Vpp, FALSE=OE}
BEGIN
	IF STATE THEN REG2:=REG2 OR 2		{SET VPP}
		ELSE REG2:=REG2 AND $FD;		{SET OE}
	PORT[BASEADDR]:=REG2;
	PCEB_LATCH2;
	DELAY(50);
END;

{*****************************************************************}
PROCEDURE PCEB_PIN3(STATE:BOOLEAN);	{SET PIN 3 RELAY}
													{TRUE=Vpp, FALSE=A15}
BEGIN
	IF STATE THEN REG2:=REG2 OR 4		{SET VPP}
		ELSE REG2:=REG2 AND $FB;		{SET A15}
	PORT[BASEADDR]:=REG2;
	PCEB_LATCH2;
	DELAY(50);
END;

{*****************************************************************}
PROCEDURE PCEB_PIN30(STATE:BOOLEAN);{SET PIN 30 RELAY}
													{TRUE=A17, FALSE=Vcc}
BEGIN
	IF STATE THEN REG2:=REG2 OR 8		{SET A17}
		ELSE REG2:=REG2 AND $F7;		{SET VCC}
	PORT[BASEADDR]:=REG2;
	PCEB_LATCH2;
	DELAY(50);
END;

{*****************************************************************}
PROCEDURE PCEB_PIN31(STATE:BOOLEAN);{SET PIN 31 MUX}
													{TRUE=PGM, FALSE=A18}
BEGIN
	IF STATE THEN REG2:=REG2 OR 32	{SET PGM}
		ELSE REG2:=REG2 AND $DF;		{SET A18}
	PORT[BASEADDR]:=REG2;
	PCEB_LATCH2;
END;

{*****************************************************************}
PROCEDURE PCEB_PIN29(STATE:BOOLEAN);{SET PIN 29 MUX}
													{TRUE=PGM, FALSE=A14}
BEGIN
	IF STATE THEN REG2:=REG2 OR 64		{SET PGM}
		ELSE REG2:=REG2 AND $BF;			{SET A14}
	PORT[BASEADDR]:=REG2;
	PCEB_LATCH2;
END;

{*****************************************************************}
PROCEDURE PCEB_PIN22(STATE:BOOLEAN);{SET PIN 22 MUX}
													{TRUE=PGM, FALSE=CE}
BEGIN
	IF STATE THEN REG2:=REG2 OR 128	{SET PGM}
		ELSE REG2:=REG2 AND $7F;		{SET CE}
	PORT[BASEADDR]:=REG2;
	PCEB_LATCH2;
END;

{*****************************************************************}
PROCEDURE PCEB_VCC(STATE:BOOLEAN);	{TURN VCC POWER TO EPROM ON & OFF}
													{TRUE=Vcc ON, FALSE=Vcc OFF}
BEGIN
	IF STATE THEN PORT[BASEADDR+2]:=PORT[BASEADDR+2] AND $F7
		ELSE PORT[BASEADDR+2]:=PORT[BASEADDR+2] OR 8;
	DELAY(50);									{ALLOW FOR RELAY SWITCHING}
END;

{*****************************************************************}
FUNCTION  PCEB_ID_MANUF:BYTE;			{GET MANUFACTURER ID CODE}
VAR V:BYTE;
BEGIN
	PCEB_STABLE;
	PCEB_SUPPLY(FALSE);
	PCEB_VCC(TRUE);
	PCEB_CE(FALSE);						{CHIP ENABLE FALSE}
	PCEB_OE(FALSE);						{OE FALSE}
	PCEB_PIN26(TRUE);						{TURN ON VPP TO A9 PIN}
	PCEB_VPP(TRUE);						{TURN VPP ON}
	V:=PCEB_GETBYTE;						{GET ID CODE}
	PCEB_VPP(FALSE);
	PCEB_STABLE;
	PCEB_PIN26(FALSE);
	PCEB_ID_MANUF:=V;
END;

{*****************************************************************}
FUNCTION  PCEB_ID_DEVICE:BYTE;		{GET DEVICE ID CODE}
VAR V:BYTE;
BEGIN
	PCEB_STABLE;
	PCEB_SUPPLY(FALSE);
	PCEB_VCC(TRUE);
	PCEB_ACLK;								{INCREMENT TO FIRST ADDRESS}
	PCEB_CE(FALSE);						{CHIP ENABLE FALSE}
	PCEB_OE(FALSE);						{OE FALSE}
	PCEB_PIN26(TRUE);						{TURN ON VPP TO A9 PIN}
	PCEB_VPP(TRUE);						{TURN VPP ON}
	V:=PCEB_GETBYTE;						{GET ID CODE}
	PCEB_VPP(FALSE);
	PCEB_STABLE;
	PCEB_PIN26(FALSE);
	PCEB_ID_DEVICE:=V;
END;

{*****************************************************************}
PROCEDURE PCEB_STABLE;           	{PUT PCEB INTO A STABLE STATE SO}
												{EPROM CAN BE INSERTED & REMOVED}
BEGIN
	PCEB_SUPPLY(FALSE);
	PCEB_VPP(FALSE);
	PCEB_ARST;
	PCEB_CE(FALSE);
	PCEB_OE(FALSE);
	PCEB_PGM(TRUE);
	PCEB_PIN26(FALSE);
	PCEB_VCC(FALSE);
	PCEB_SETBYTE(0);
END;


{*****************************************************************}
BEGIN
END.